@@ -78,7 +78,7 @@ pathlib = _LazyModule.load('pathlib', 'pathlib')
 platform = _LazyModule.load('platform', 'platform')
 functools = _LazyModule.load('functools', 'functools')
 subprocess = _LazyModule.load('subprocess', 'subprocess')
-__version__ = '0.2.1'
+__version__ = '0.2.2'
 
 
 @functools.lru_cache(1)
@@ -348,7 +348,8 @@ def bash_completions(prefix, line, begidx, endidx, env=None, paths=None,
     quote_paths : callable, optional
         A functions that quotes file system paths. You shouldn't normally need
         this as the default is acceptable 99+% of the time. This function should
-        a set of the new paths and a boolean for whether the paths were quoted.
+        return a set of the new paths and a boolean for whether the paths were
+        quoted.
 
     Returns
     -------
@@ -418,7 +419,7 @@ def bash_completions(prefix, line, begidx, endidx, env=None, paths=None,
     if '-o nospace' in complete_stmt:
         out = set([x.rstrip() for x in out])
 
-    return out, len(prefix) - strip_len
+    return out, max(len(prefix) - strip_len, 0)
 
 
 def bash_complete_line(line, return_line=True, **kwargs):
@@ -983,6 +984,19 @@ def PATTERN_NEED_QUOTES():
     return re.compile(pattern)
 
 
+def cd_in_command(line):
+    """Returns True if "cd" is a token in the line, False otherwise."""
+    lexer = builtins.__xonsh_execer__.parser.lexer
+    lexer.reset()
+    lexer.input(line)
+    have_cd = False
+    for tok in lexer:
+        if tok.type == 'NAME' and tok.value == 'cd':
+            have_cd = True
+            break
+    return have_cd
+
+
 def _path_from_partial_string(inp, pos=None):
     if pos is None:
         pos = len(inp)
@@ -1258,7 +1272,7 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
     if tilde in prefix:
         home = os.path.expanduser(tilde)
         paths = {s.replace(home, tilde) for s in paths}
-    if cdpath:
+    if cdpath and cd_in_command(line):
         _add_cdpaths(paths, prefix)
     paths = set(filter(filtfunc, paths))
     paths, _ = _quote_paths({_normpath(s) for s in paths},
@@ -1576,7 +1590,6 @@ def default_completers():
         ('completer', complete_completer),
         ('skip', complete_skipper),
         ('pip', complete_pip),
-        ('xpip', complete_pip),
         ('cd', complete_cd),
         ('rmdir', complete_rmdir),
         ('xonfig', complete_xonfig),
