@@ -47,6 +47,8 @@ ON_WINDOWS = LazyBool(lambda: platform.system() == 'Windows',
 """``True`` if executed on a native Windows platform, else ``False``. """
 ON_CYGWIN = LazyBool(lambda: sys.platform == 'cygwin', globals(), 'ON_CYGWIN')
 """``True`` if executed on a Cygwin Windows platform, else ``False``. """
+ON_MSYS = LazyBool(lambda: sys.platform == 'msys', globals(), 'ON_MSYS')
+"""``True`` if executed on a MSYS Windows platform, else ``False``. """
 ON_POSIX = LazyBool(lambda: (os.name == 'posix'), globals(), 'ON_POSIX')
 """``True`` if executed on a POSIX-compliant platform, else ``False``. """
 ON_FREEBSD = LazyBool(lambda: (sys.platform.startswith('freebsd')),
@@ -114,14 +116,14 @@ def pygments_version():
 
 @functools.lru_cache(1)
 def has_prompt_toolkit():
-    """ Tests if the `prompt_toolkit` is available. """
+    """Tests if the `prompt_toolkit` is available."""
     spec = importlib.util.find_spec('prompt_toolkit')
     return (spec is not None)
 
 
 @functools.lru_cache(1)
 def ptk_version():
-    """ Returns `prompt_toolkit.__version__` if available, else ``None``. """
+    """Returns `prompt_toolkit.__version__` if available, else ``None``."""
     if has_prompt_toolkit():
         import prompt_toolkit
         return getattr(prompt_toolkit, '__version__', '<0.57')
@@ -200,7 +202,7 @@ def expanduser():
 def windows_expanduser(path):
     """A Windows-specific expanduser() function for xonsh. This is needed
     since os.path.expanduser() does not check on Windows if the user actually
-    exists. This restircts expanding the '~' if it is not followed by a
+    exists. This restricts expanding the '~' if it is not followed by a
     separator. That is only '~/' and '~\' are expanded.
     """
     if not path.startswith('~'):
@@ -343,7 +345,7 @@ def windows_bash_command():
         else:
             # Check if Bash is from the "Windows Subsystem for Linux" (WSL)
             # which can't be used by xonsh foreign-shell/completer
-            bash_works = 'pc-linux-gnu' not in out.splitlines()[0]
+            bash_works = out and 'pc-linux-gnu' not in out.splitlines()[0]
 
         if bash_works:
             wbc = bash_on_path
@@ -361,7 +363,7 @@ def windows_bash_command():
 
 if ON_WINDOWS:
     class OSEnvironCasePreserving(collections.MutableMapping):
-        """ Case-preseving wrapper for os.environ on Windows.
+        """ Case-preserving wrapper for os.environ on Windows.
             It uses nt.environ to get the correct cased keys on
             initialization. It also preserves the case of any variables
             add after initialization.
@@ -414,7 +416,7 @@ if ON_WINDOWS:
 
 @lazyobject
 def os_environ():
-    """This dispatches to the correct, case-senstive version of os.envrion.
+    """This dispatches to the correct, case-sensitive version of os.environ.
     This is mainly a problem for Windows. See #2024 for more details.
     This can probably go away once support for Python v3.5 or v3.6 is
     dropped.
@@ -427,7 +429,7 @@ def os_environ():
 
 @functools.lru_cache(1)
 def bash_command():
-    """Determines the command for Bash on the current plaform."""
+    """Determines the command for Bash on the current platform."""
     if ON_WINDOWS:
         bc = windows_bash_command()
     else:
@@ -440,7 +442,7 @@ def BASH_COMPLETIONS_DEFAULT():
     """A possibly empty tuple with default paths to Bash completions known for
     the current platform.
     """
-    if ON_LINUX or ON_CYGWIN:
+    if ON_LINUX or ON_CYGWIN or ON_MSYS:
         bcd = ('/usr/share/bash-completion/bash_completion', )
     elif ON_DARWIN:
         bcd = ('/usr/local/share/bash-completion/bash_completion',  # v2.x
@@ -458,7 +460,7 @@ def BASH_COMPLETIONS_DEFAULT():
 
 @lazyobject
 def PATH_DEFAULT():
-    if ON_LINUX or ON_CYGWIN:
+    if ON_LINUX or ON_CYGWIN or ON_MSYS:
         if linux_distro() == 'arch':
             pd = ('/usr/local/sbin',
                   '/usr/local/bin', '/usr/bin', '/usr/bin/site_perl',
@@ -485,10 +487,14 @@ def PATH_DEFAULT():
 @lazyobject
 def LIBC():
     """The platform dependent libc implementation."""
+    global ctypes
     if ON_DARWIN:
+        import ctypes.util
         libc = ctypes.CDLL(ctypes.util.find_library("c"))
     elif ON_CYGWIN:
         libc = ctypes.CDLL('cygwin1.dll')
+    elif ON_MSYS:
+        libc = ctypes.CDLL('msys-2.0.dll')
     elif ON_BSD:
         try:
             libc = ctypes.CDLL('libc.so')
